@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (c) 1998-2006 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (c) 1998-2007 Michal Trojnara <Michal.Trojnara@mirt.net>
  *                 All Rights Reserved
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef COMMON_H
@@ -24,9 +24,11 @@
 /**************************************** Common constants */
 
 #ifndef VERSION
-#define VERSION "4.18"
+#define VERSION "undefined"
 #endif
 
+#define LIBWRAP_CLIENTS 5
+
 #ifdef OPTIMIZE_SCALABILITY
 
 /* CPU stack size */
@@ -36,9 +38,6 @@
 /* I/O buffer size */
 #define BUFFSIZE        2048
 
-/* Length of strings (including the terminating '\0' character) */
-#define STRLEN          128
-
 #else /* OPTIMIZE_SCALABILITY */
 
 /* CPU stack size */
@@ -47,11 +46,12 @@
 /* I/O buffer size */
 #define BUFFSIZE        16384
 
-/* Length of strings (including the terminating '\0' character) */
-#define STRLEN          256
-
 #endif /* OPTIMIZE_SCALABILITY */
 
+/* Length of strings (including the terminating '\0' character) */
+/* It can't be lower than 256 bytes or NTLM authentication will break */
+#define STRLEN          256
+
 /* IP address and TCP port textual representation length */
 #define IPLEN           128
 
@@ -66,6 +66,7 @@
 
 #ifdef USE_WIN32
 #define USE_IPv6
+/* #define USE_FIPS */
 #endif
 
 #ifdef _WIN32_WCE
@@ -310,22 +311,43 @@ extern char *sys_errlist[];
 /**************************************** OpenSSL headers */
 
 #ifdef HAVE_OPENSSL
+
+#define OPENSSL_THREAD_DEFINES
+#include <openssl/opensslconf.h>
+#if !defined(OPENSSL_THREADS) && defined(USE_PTHREAD)
+#error OpenSSL library compiled without thread support
+#endif /* !OPENSSL_THREADS && USE_PTHREAD */
+
 #include <openssl/lhash.h>
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <openssl/crypto.h> /* for CRYPTO_* and SSLeay_version */
 #include <openssl/rand.h>
+#include <openssl/md4.h>
+#include <openssl/des.h>
+
 #ifdef HAVE_OSSL_ENGINE_H
 #include <openssl/engine.h>
-#endif
+#endif /* HAVE_OSSL_ENGINE_H */
+
 #if SSLEAY_VERSION_NUMBER >= 0x00907000L
 #include <openssl/ocsp.h>
 #endif /* OpenSSL-0.9.7 */
+
+#ifdef USE_FIPS
+#include <openssl/fips.h>
+#include <openssl/fips_rand.h>
+#endif /* USE_FIPS */
+
 #else /* HAVE_OPENSSL */
+
 #include <lhash.h>
 #include <ssl.h>
 #include <err.h>
-#include <crypto.h> /* for CRYPTO_* and SSLeay_version */
+#include <crypto.h>
+#include <md4.h>
+#include <des.h>
+
 #endif /* HAVE_OPENSSL */
 
 /**************************************** Other defines */
@@ -337,7 +359,7 @@ extern char *sys_errlist[];
     (dst[STRLEN-1]='\0', strncat((dst), (src), STRLEN-strlen(dst)-1))
 /* change all non-printable characters to '.' */
 #define safestring(s) \
-    do {unsigned char *p; for(p=(s); *p; p++) \
+    do {unsigned char *p; for(p=(unsigned char *)(s); *p; p++) \
         if(!isprint((int)*p)) *p='.';} while(0)
 /* change all unsafe characters to '.' */
 #define safename(s) \
