@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (c) 1998-2003 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (c) 1998-2007 Michal Trojnara <Michal.Trojnara@mirt.net>
  *                 All Rights Reserved
  *
  *   Based on a Public Domain code by Tatu Ylonen <ylo@cs.hut.fi>
@@ -17,7 +17,7 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *   In addition, as a special exception, Michal Trojnara gives
  *   permission to link the code of this program with the OpenSSL
@@ -50,6 +50,10 @@
 #include <pty.h>
 #endif /* HAVE_PTY_H */
 
+#ifdef HAVE_LIBUTIL_H
+#include <libutil.h>
+#endif /* HAVE_LIBUTIL_H */
+
 #ifndef O_NOCTTY
 #define O_NOCTTY 0
 #endif /* O_NOCTTY */
@@ -62,7 +66,7 @@
  */
 
 int pty_allocate(int *ptyfd, int *ttyfd, char *namebuf, int namebuflen) {
-#if defined(HAVE_OPENPTY) || defined(BSD4_4)
+#if defined(HAVE_OPENPTY) || defined(BSD4_4) && !defined(__INNOTEK_LIBC__)
     /* openpty(3) exists in OSF/1 and some other os'es */
     char buf[64];
     int i;
@@ -121,7 +125,7 @@ int pty_allocate(int *ptyfd, int *ttyfd, char *namebuf, int namebuflen) {
     }
     pts = ptsname(ptm);
     if (pts == NULL)
-        log(LOG_ERR, "Slave pty side name could not be obtained");
+        s_log(LOG_ERR, "Slave pty side name could not be obtained");
     safecopy(namebuf, pts);
     *ptyfd = ptm;
 
@@ -152,7 +156,7 @@ int pty_allocate(int *ptyfd, int *ttyfd, char *namebuf, int namebuflen) {
     }
     name = ttyname(*ptyfd);
     if (!name) {
-        log(LOG_ERR, "Open of /dev/ptc returns device for which ttyname fails");
+        s_log(LOG_ERR, "Open of /dev/ptc returns device for which ttyname fails");
         return -1;
     }
     safecopy(namebuf, name);
@@ -214,10 +218,10 @@ int pty_allocate(int *ptyfd, int *ttyfd, char *namebuf, int namebuflen) {
 
 void pty_release(char *tty_name) {
     if(chown(tty_name, (uid_t)0, (gid_t)0)<0)
-        log(LOG_DEBUG, "chown %.100s 0 0 failed: %.100s", tty_name,
+        s_log(LOG_DEBUG, "chown %.100s 0 0 failed: %.100s", tty_name,
             strerror(get_last_socket_error()));
     if(chmod(tty_name, (mode_t)0666)<0)
-        log(LOG_DEBUG, "chmod %.100s 0666 failed: %.100s", tty_name,
+        s_log(LOG_DEBUG, "chmod %.100s 0666 failed: %.100s", tty_name,
             strerror(get_last_socket_error()));
 }
 
@@ -243,12 +247,12 @@ void pty_make_controlling_tty(int *ttyfd, char *tty_name) {
      */
     fd = open("/dev/tty", O_RDWR | O_NOCTTY);
     if (fd >= 0) {
-        log(LOG_ERR, "Failed to disconnect from controlling tty");
+        s_log(LOG_ERR, "Failed to disconnect from controlling tty");
         close(fd);
     }
     /* Make it our controlling tty. */
 #ifdef TIOCSCTTY
-    log(LOG_DEBUG, "Setting controlling tty using TIOCSCTTY");
+    s_log(LOG_DEBUG, "Setting controlling tty using TIOCSCTTY");
     /*
      * We ignore errors from this, because HPSUX defines TIOCSCTTY, but
      * returns EINVAL with these arguments, and there is absolutely no
